/* Copyright 2009 Palm, Inc.  All rights reserved. */

var MediaKeyControlHandler = Class.create(
{
	initialize: function(musicPlayer)
	{
		this.musicPlayer = musicPlayer;
		
		this.headsetEvents = this.registerForHeadsetEvents(this.headsetCallback.bind(this));
		
		this.bluetoothEvents = this.registerForAVRCPEvents(this.bluetoothCallback.bind(this));
	},
	
	
	lockVolumeKeys: function(callback)
	{
		
		var parameters = {};
		parameters.subscribe = true;
		parameters.foregroundApp = true;
		
		return new Mojo.Service.Request(
				"palm://com.palm.audio/media",
				{
					method: 'lockVolumeKeys',					
					onSuccess: callback,
					parameters: parameters
				}
			);
	},
	
	registerForHeadsetEvents: function(callback)
	{
		var parameters = {};
		parameters.subscribe = true;

		return new Mojo.Service.Request(
				"palm://com.palm.keys/headset",
				{
					method: 'status',
					onSuccess: callback,
					parameters: parameters

				}, true
			);
	},
	
	registerForAVRCPEvents: function(callback)
	{
		var parameters = {};
		parameters.subscribe = true;

		return new Mojo.Service.Request(
				"palm://com.palm.keys/media",
				{
					method: 'status',
					onSuccess: callback,
					parameters: parameters

				}, true
			);
	},
	
	
	headsetCallback: function(event)
	{
		var key = event.key;

		if (key !== undefined) {
		
			if (key === "headset_button") {
				var state = event.state;
				
				if (state === "single_click") {
					this.keyPressCallback("togglePausePlay");
				//setDoubleClickTimer();
				}
				else 
					if (state === "double_click") {
						this.keyPressCallback("nextAndPlay");
					// put this back in when NOV-79034 is fixed
					//clearDoubleClickTimer();
					//handleMediaKeyEvent("next");
					}
			}
		}	
	},
	
	bluetoothCallback : function(event){
		var state = event.state;
		
		if (state){

			if (state === "up"){
				var key = event.key;			
	
				switch (key){
					// this looks strange, but there may be events we need to translate
					
					case "next":
						this.keyPressCallback('next');//handleMediaKeyEvent("next");
						break;
					case "prev":
						this.keyPressCallback('prev');//handleMediaKeyEvent("prev");
						break;
					case "pause":
						this.keyPressCallback('pause');//handleMediaKeyEvent("pause");
						break;
					case "play":
						this.keyPressCallback('play');//handleMediaKeyEvent("play");
						break;
					case "stop":
						this.keyPressCallback('stop');//handleMediaKeyEvent("stop");
						break;
					case "repeat-all":
						this.keyPressCallback('repeat-all');//handleMediaKeyEvent("repeat-all");
						break;
					case "repeat-track":
						this.keyPressCallback('repeat-track');//handleMediaKeyEvent("repeat-track");
						break;
					case "repeat-none":
						this.keyPressCallback('repeat-none');//handleMediaKeyEvent("repeat-none");
						break;
					case "shuffle-on":
						this.keyPressCallback('shuffle-on');//handleMediaKeyEvent("shuffle-on");
						break;
					case "shuffle-off":
						this.keyPressCallback('shuffle-off');//handleMediaKeyEvent("shuffle-off");
						break;
				}
	
			}
		}
	},
	
	
	keyPressCallback: function(key){

		switch (key) {
			
			case "next":
				if (this.musicPlayer.playing){
					this.musicPlayer.lastPlayingState = true;
				} 

				this.musicPlayer.nextWPA();
				break;
				
			case "nextAndPlay":
				// start playing automatically when the song is switched
				// this is needed to work around NOV-79034
				
				this.musicPlayer.lastPlayingState = true;
				this.musicPlayer.nextWPA();
				this.musicPlayer.play();
				break;
				
			case "prev":
				if (this.musicPlayer.playing){
					this.musicPlayer.lastPlayingState = true;
				} 
				this.musicPlayer.previousWPA();
				break;
				
			case "pause":
				this.musicPlayer.pause();
				break;
				
			case "stop":
				this.musicPlayer.pause();
				break;
				
			case "play":
				this.musicPlayer.play();
				break;
				
			case "togglePausePlay":
				if (this.musicPlayer.playing)
					this.musicPlayer.pause();
				else
					this.musicPlayer.play();
				break;
		}
	}

});

