/**
 * @author grisha
 */

var musicWorkerSingleton = (function(){
    var instance = null;
	
	var MusicWorker = Class.create({
		
		initialize: function(name, sound) {
			
			this.lastPlayingState = false;
			
			
			this.CPManager = CPManagerSingleton.getInstance();
			
			this.loadedPlaylist = [];
			this.playlistOrder = [];
			this.activeOrderNumber = 0;
			
			this.headsetKeyHandler = new MediaKeyControlHandler(this);
			this.utilities = utilsManagerSingleton.getInstance();
			this.unsetNowplayingScene();
			
			this.playing = false;
		
			//get shuffle and repeat
			this.shuffle = prefsManagerSingleton.getInstance().getBooleanCookie("amigoShuffle");
			this.repeat = prefsManagerSingleton.getInstance().getBooleanCookie("amigoRepeat");
				
			
		  	//Load MediaExtension
			this.libs = MojoLoader.require({ name: "mediaextension", version: "1.0"});
			
			//Create audio object
			this.audioPlayer = new Audio(); 
			
			// Instantiate the MediaExtension object and set audioClass to 'media'
			this.extObj = this.libs.mediaextension.MediaExtension.getInstance(this.audioPlayer);
			this.extObj.audioClass = "media"; 
			
			//Set play to false. We don't want it.
			this.audioPlayer.autoplay  = false;
			
			//Set needed event listeners
			
			//this.audioPlayer.addEventListener('x-palm-disconnect', this.mediaServerDisconnect.bind(this));
			//this.audioPlayer.addEventListener('x-palm-watchdog-triggered', this.mediaServerWatchdog.bind(this));
			this.audioPlayer.addEventListener('ended', this.handleCurrentSongEnded.bind(this));
			//this.audioPlayer.addEventListener('loadstart', this._elementLoadStart.bind(this));
			this.audioPlayer.addEventListener('canplaythrough', this.handleCanPlayThrough.bind(this));
			this.audioPlayer.addEventListener('pause', this.handlePause.bind(this));
			this.audioPlayer.addEventListener('play', this.handlePlay.bind(this));
			//this.audioPlayer.addEventListener('error', this.mediaServerError.bind(this));
			
			this.audioPlayer.addEventListener('timeupdate', this.handleTimeUpdate.bind(this));
			
			this.playTimer = -1;
			
			this.secondsPlayedInCurrentSong = 0;
	  	},
	  	
	  	
		
		playTimerTick : function(){
			this.secondsPlayedInCurrentSong = this.secondsPlayedInCurrentSong + 1;
			window.clearTimeout(this.playTimer);
			this.playTimer = window.setTimeout(function() {
				this.playTimerTick(); }.bind(this), 1000);
		},
		
		cleanDashboard: function(){
			var appController = Mojo.Controller.getAppController();
			//var dashboardStage = Mojo.Controller.getAppController().getStageProxy(DashboardStageName);
			appController.closeStage(AppAssistant.DashboardStageName);
		},
		
		updateDashboard: function(){
			
			var stageController = Mojo.Controller.getAppController().getStageController(AppAssistant.MainStageName);
			
			if (this.loadedPlaylist.length > 0 && !AppAssistant.hasFocus && !AppAssistant.dockMode && stageController) {
				var dataObject = {};
				
				dataObject = this.getCurrentSongData();
				dataObject.playing = this.playing;
				dataObject.artwork = this.utilities.getPictureURL(this.getCurrentSong());
				
				var appController = Mojo.Controller.getAppController();
				var dashboardStageController = appController.getStageController(AppAssistant.DashboardStageName);
				
				
				//if dashboard stage is active then push update to it's active scene
				if (dashboardStageController) {
					dashboardStageController.delegateToSceneAssistant("updateDashboard", dataObject);
				}
				
				
				//create the dashboard stage
				else {
					var pushDashboard = function(stageController){
						stageController.pushScene('dashboard', dataObject);
					};
					
					var self = this;
					
					appController.createStageWithCallback({
						name: AppAssistant.DashboardStageName,
						lightweight: true,
						clickableWhenLocked: true
					}, pushDashboard.bind(this), 'dashboard');
				}
			}
		},
		
		toggleShuffle: function(){
			if (this.shuffle == true) {
				this.shuffle = false;
				prefsManagerSingleton.getInstance().unsetBooleanCookie("amigoShuffle");
				this.undoShuffle();
			}
			else {
				this.shuffle = true;
				prefsManagerSingleton.getInstance().setBooleanCookie("amigoShuffle");
				this.doShuffle();
			}
			if (this.nowplayingScene){
				this.nowplayingScene.updateShuffleAndRepeat(this.shuffle,this.repeat);
			}
		},
		
		toggleRepeat: function(){
			if (this.repeat) {
				this.repeat = false;
				prefsManagerSingleton.getInstance().unsetBooleanCookie("amigoRepeat");
			}
			else{
				this.repeat = true;
				prefsManagerSingleton.getInstance().setBooleanCookie("amigoRepeat");
			}
			if (this.nowplayingScene){
				this.nowplayingScene.updateShuffleAndRepeat(this.shuffle,this.repeat);
			}			
		},
		

		
		handleShakeEnd: function(){
			if (this.nowplayingScene) {
				this.nowplayingScene.controller.stageController.getAppController().playSoundNotification("vibrate", "");
				if (this.shuffle) {
					this.shuffle = false;
					this.toggleShuffle();
					this.next();
					this.nowplayingScene.updateSongInfo(true);
					//this.handleCurrentSongEnded();
				}
				else {
					var randomnumber = Math.floor(Math.random() * this.playlistOrder.length);
					this.activeOrderNumber = randomnumber;
					if (this.nowplayingScene) {
						this.next();
						this.nowplayingScene.updateSongInfo(true);
					//this.handleCurrentSongEnded();
					}
				}
			}
		},
		
		handleCurrentSongEnded: function(){
			if (this.isCurrentLast() && !this.repeat) {
				this.pause();
			}
			else {
				if (this.nowplayingScene != null) {
					this.nowplayingScene.nextTapped(null);
				}
				else {
					this.next();
				}
			}
			this.updateDashboard();
		},
		
		handleTimeUpdate: function(){
			if (this.nowplayingScene != null){
				this.nowplayingScene.updateTimeAndSlider(
					this.getDuration(),
					//this.secondsPlayedInCurrentSong,
					this.audioPlayer.currentTime,
					this.timeFormat (this.getDuration()),
					this.timeFormat (this.audioPlayer.currentTime));
			}
			
			if (this.getDuration() < this.audioPlayer.currentTime)
			{
				this.handleCurrentSongEnded(null);
			}
		},
		
		handlePlay: function(){
			//Mojo.Log.info(this.playing+'HPlay');
			this.playTimer = window.setTimeout(function() {
				this.playTimerTick(); }.bind(this), 1000);
			
			this.playing = !this.audioPlayer.paused;
			this.handlePlayPause();
		},

		handlePause: function(){
			//Mojo.Log.info(this.playing+'HPause');
			window.clearTimeout(this.playTimer);
			this.playing = false;
			this.handlePlayPause();
		},		
		
		handlePlayPause: function(){
			//Mojo.Log.info(this.playing+'HPP');
			if (this.nowplayingScene != null){
				this.nowplayingScene.updatePlayPause(this.playing);
			}
			
			this.updateDashboard();
		},		
		
		timeFormat: function (val){
			if (isNaN(val)) val = 0;
			var mins = parseInt(String(val/60));
			if (mins<10)
				mins = "0"+String(mins);
			var secs = parseInt(String(val%60));
			if (secs<10)
				secs = "0"+String(secs);
			return mins+":"+secs;				
		},
		
		loadPlaylist: function (playlist, activeSongN)
		{
			if (!this.playing || this.loadedPlaylist.length == 0 || Object.toJSON(playlist[activeSongN]) != Object.toJSON(this.getCurrentSong())) {
				
				this.loadedPlaylist = [];
				this.playlistOrder = [];
				
				this.loadedPlaylist = playlist;
				
				this.activeOrderNumber = activeSongN;
				
				for (var i = 0; i < this.loadedPlaylist.length; i++) {
					this.playlistOrder[i] = i;
				}
				
				if (this.shuffle == true) {
					this.doShuffle();
				}
				
				this.saveNowPlayingPlaylist();
				this.loadNewSong(this.playlistOrder[this.activeOrderNumber]);
			}
		},
		
		doShuffle: function(){
			var firstValue;
			firstValue = this.playlistOrder.splice(this.activeOrderNumber,1);
			this.playlistOrder = this.shuffleArray(this.playlistOrder);
			this.playlistOrder.unshift(firstValue);
			this.activeOrderNumber = 0;
		},
		
		undoShuffle: function(){
			var curSong = this.getCurrentSong();
			utilsManagerSingleton.getInstance().qsort(this.playlistOrder);
			//this.playlistOrder.sort(function(a,b){return a - b});
			var i;
			for (i=0;i<this.playlistOrder.length;i++){
				if (this.loadedPlaylist[this.playlistOrder[i]] == (curSong)){
					this.activeOrderNumber = i;
				}
			}
		},
		
		loadNewSong: function(index){
			
			this.lastPlayingState = this.playing;
			//scrobble old song if needed
			prefsManagerSingleton.getInstance().setCookie(AppAssistant.NOWPLAYING_PLAYLIST_INDEX,{
					'artist': this.getCurrentSong().artist,
					'title': this.getCurrentSong().title,
					'album': this.getCurrentSong().album
				});
				
			this.audioPlayer.src = this.loadedPlaylist[index].path;
			this.audioPlayer.load();
			this.secondsPlayedInCurrentSong = 0;		
		},
		
		
		handleCanPlayThrough : function(){
			if (this.lastPlayingState){
				this.play();
			}
			else{
				//this.pause();
			}
			
			this.submitToSEWatch();
			
		},
		
		next: function(){
			this.scrobToLastFM();
			this.activeOrderNumber = this.getNextOrderIndex();
			this.loadNewSong(this.playlistOrder[this.activeOrderNumber]);
		},
		
		//next with possible animation
		nextWPA: function(){
			if (this.nowplayingScene){
				//Mojo.Log.info('here');
				this.nowplayingScene.nextTapped();
			}
			else{
				this.next();
				this.updateDashboard();
			}
			
		},
		
		//previous with possible animation
		previousWPA: function(){
			if (this.nowplayingScene){
				this.nowplayingScene.previousTapped();
			}
			else{
				this.previous();
				this.updateDashboard();
			}
			
		},		
		
		previous: function(){
			this.scrobToLastFM();
			this.activeOrderNumber = this.getPreviousOrderIndex();
			this.loadNewSong(this.playlistOrder[this.activeOrderNumber]);
		},
		
		getNextOrderIndex: function(){
			
			if (this.isCurrentLast()){
				if (this.repeat) {
					return 0;
				}
				else {
					return this.activeOrderNumber;
				}
			}
			else{
				return (this.activeOrderNumber+1);
			}
		},
		
		getPreviousOrderIndex: function(){
			if (this.isCurrentFirst()){
				if (this.repeat) {
					return this.playlistOrder.length - 1;
				}
				else {
					return 0;
				}	
			}
			else{
				return (this.activeOrderNumber-1);
			}
		},
	
		play: function() {
			this.playing = true;
			//this.handlePlayPause();
			this.audioPlayer.play();
		},
	  
		pause: function() {
			this.playing = false;
			//this.handlePlayPause();
			this.audioPlayer.pause();
		},
		  
		rewind: function() {
			this.audioPlayer.currentTime = 0;
		},
		
		getSong: function(index){
			return this.loadedPlaylist[this.playlistOrder[index]];
		},
		
		getCurrentSong: function(){
			return this.getSong(this.activeOrderNumber);
		},
		  
		getCurrentArtwork: function(){
			var song = this.loadedPlaylist[this.playlistOrder[this.activeOrderNumber]];
			return this.utilities.getSizedAlbumArtString(song.path,song.songPictureUrl,320,320);
		},
		
		getNextArtwork: function(){//this.getNextOrderIndex()
			var song = this.loadedPlaylist[this.playlistOrder[this.getNextOrderIndex()]];
			return this.utilities.getSizedAlbumArtString(song.path,song.songPictureUrl,320,320);
		},
		
		getPreviousArtwork: function(){//this.getPreviousOrderIndex()
			var song = this.loadedPlaylist[this.playlistOrder[this.getPreviousOrderIndex()]];
			return this.utilities.getSizedAlbumArtString(song.path,song.songPictureUrl,320,320);
		},		
	
		getCurrentSongData: function(){
			if (this.loadedPlaylist.length == 0) {
				return null;
			}
			else {
				var song = this.loadedPlaylist[this.playlistOrder[this.activeOrderNumber]];
				return {
					title: song.title,
					artist: song.artist,
					album: song.album
				};
			}
		},
		
		shuffleArray: function(v){
    		for (var j, x, i = v.length; i; j = parseInt(Math.random() * i), x = v[--i], v[i] = v[j], v[j] = x);
    		return v;
		},
		
		
		setNowplayingScene: function(nps){
			this.nowplayingScene = nps;
			this.nowplayingScene.updateShuffleAndRepeat(this.shuffle,this.repeat);
			this.nowplayingScene.updatePlayPause(this.playing);
		},
		
		unsetNowplayingScene: function(){
			if (this.nowplayingScene) {
				this.nowplayingScene = null;
			}
		},
		
		nowplayingToggleControls: function(value){
			if (this.nowplayingScene) {
				this.nowplayingScene.toggleControls(value);
			}
		},
		
		isCurrentLast: function(){
			if (this.activeOrderNumber == this.playlistOrder.length-1)
				return true;
			else	
				return false;	
		},
		
		isCurrentFirst: function(){
			if (this.activeOrderNumber == 0)
				return true;
			else
				return false;	
		},
		
		scrubTo: function(val){
			this.audioPlayer.currentTime = val*this.getDuration()/100;
		},
		
		
		saveNowPlayingPlaylist: function(){
			this.CPManager.deleteCP(AppAssistant.NOWPLAYING_PLAYLIST);

			var model = {items: []};

			for (var i=0;i<this.loadedPlaylist.length;i++){
				var song = this.loadedPlaylist[i];
				model.items.push({'artist':song.artist, 'album':song.album, 'title':song.title});
			}
			
			//Mojo.Log.info(Object.toJSON(model));
					
			this.CPManager.saveCP(AppAssistant.NOWPLAYING_PLAYLIST, model);
		},
		
		
		scrobToLastFM : function(){
			if (this.audioPlayer.src != null && this.audioPlayer.src.length>0){
				AppAssistant.lastFMManager.scrobbleExternal(
					this.getDuration(),
					this.secondsPlayedInCurrentSong,
					this.getCurrentSong(),
					function(r){}
				);
			}			
		},
		
		
		loadNowPlayingPlaylist: function(sc,callback){
			this.CPManager.getCP(AppAssistant.NOWPLAYING_PLAYLIST,
				function(np)
				{
					var nowplayinglist = np;
					if (nowplayinglist.length > 0) 
					{
						AppAssistant.dbManager.listSongs(sc,null,null,null,false, function(response)					
						{							
							var songList = response.results;
							var newlist = [];
							for (var j = 0; j < nowplayinglist.length; j++) {
								//Mojo.Log.info(Object.toJSON(nowplayinglist[j].artist + ' ' + nowplayinglist[j].album + ' '+nowplayinglist[j].title));
																
								for (var i = 0; i < songList.length; i++) {												
									if (songList[i].artist == nowplayinglist[j].artist &&
										songList[i].album == nowplayinglist[j].album &&
										songList[i].title == nowplayinglist[j].title) {
											//Mojo.Log.info(Object.toJSON(songList[i]));
											newlist.push(songList[i]);
											break;
									}
								}
							}
							var currentSong = prefsManagerSingleton.getInstance().getCookie(AppAssistant.NOWPLAYING_PLAYLIST_INDEX)
							var index = 0;
							for (var i = 0; i < newlist.length; i++) {
								if (newlist[i].title == currentSong.title &&
								newlist[i].album == currentSong.album &&
								newlist[i].artist == currentSong.artist) {
									index = i;
									break;
								}
								
							}
							callback(0);
							this.loadPlaylist(newlist, index);
							this.pause();
							if (this.nowplayingScene) {
								this.nowplayingScene.updateSongInfo(true);
							}
						}.bind(this)); //end requesting all songs
					}//end the length if
					else 
					{
						callback(0);
					}
				}.bind(this)); //end requesting list
				
		},//end load nowplaying list
		
		getDuration : function()
		{
			if (AppAssistant.isTrial)
				return this.audioPlayer.duration*2/3;
			else
				return this.audioPlayer.duration;
		},
		
		
		submitToSEWatch : function()
		{
            //Patch report Title to BT-Watch SE MBW-150    to report new Mail to BT-watch SE MBW-150
        
        }
	});
	
	var DashboardStageName = "dashboardStage";
  
 	return new function(){
        this.getInstance = function(){
            if (instance == null) {
                instance = new MusicWorker();
                instance.constructor = null;
            }

            return instance;
        };
		this.clearInstance = function(){
			if (instance){
				instance.cleanUp();
				instance = null;
			}
		};
		this.getExistingInstance = function(){
			return instance;
		};
    };
})();
